#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# MONK automated test framework ssh connection handling
#
# Copyright (C) 2012 DResearch Fahrzeugelektronik GmbH
# Written and maintained by Thilo Fromm <fromm@dresearch-fe.de>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version
# 2 of the License, or (at your option) any later version.
#

""" Package for the SSH / SCP Connection class """

import socket
import sys
import libssh2
import os
import tempfile
import logging


class SshConn(object):
    """ The SSH connection class abstracts an SSH connection to a device. 
        It implements remote command execution as well as uploading/downloading 
        of files, and also can display the contents of a remote file.
    """
    
    def __init__(self, host, login = ("root", "")):
        """ Create a new instance of the SSH connection class. 

            :param host:    host name or IP address of the remote host
            :param login:   tuple of (username, pass)
        """
        self._logger = logging.getLogger(__name__)
        self._host = host
        self._login = login


    def _session(self):
        """ Session property. This is the current SSH session. It will be 
            created when needed.

            :return: the session (which will be created if it isn't there yet)
        """
        self._logger.info("Opening session for user %s to %s" 
                           % (self._login[0], self._host))
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.connect( (self._host, 22) )
        sock.setblocking(1)

        self.__session = libssh2.Session() # pylint: disable-msg=W0201
        self.__session.startup(sock)
        self.__session.userauth_password(self._login[0], self._login[1])
        return self.__session


    def cmd(self, cmd):
        """ Run a command on the remote device.

            :param cmd: the command to run
            :return:    tuple of (retcode, command_output) 
        """
        self._logger.info("Executing command [%s]" % cmd)
        s = self._session()
        chan = s.open_session()
        # fixup: explicitly set PATH since we missed /usr/sbin
        # in the default env at some point.
        ret = chan.execute("export PATH=\"$PATH:/bin:/sbin:/usr/bin"
                           + ":/usr/sbin:/usr/local/bin:/usr/local/sbin\";"
                           +  cmd + " 2>&1")
        if ret == 0:
            buf = ""
            while True:
                ret = chan.read(1024)
                if ret == "" or ret is None:
                    break
                buf += ret
            chan.close()
            rcd = chan.exit_status()
            self._logger.info("Command [%s] ret: #%d" % (cmd, rcd))
            self._logger.debug("Command [%s] output:\n[%s]" % (cmd, buf))
            return rcd, buf
        else:
            raise Exception(
                "Remote execution of %s failed; ret %s, session error %s." 
                    % (cmd, ret, s.last_error()) )


    def get(self, local_file, remote_path):
        """ Get a file from the remote system.

            :param local_file:  file-like object for storing the file
            :param remote_path: path to the file (including file name) 
                                on the remote system
            :return:            None 
        """
        self._logger.info("Receiving file [%s] from device" % (remote_path))
        start_offs = local_file.tell()
        chan = self._session().scp_recv(remote_path)
        local_file.write(chan.read())
        chan.close()
        size = local_file.tell() - start_offs
        self._logger.info("Received [%s], %s bytes, from device" 
                                % (remote_path, size))


    def cat(self, remote_path):
        """ Return a buffer containing the contents of a remote file.

            :param remote_path: path to the file (including file name) 
                                on the remote system
            :return:            buffer containing the file's contents 
        """
        tfl = tempfile.TemporaryFile()
        self.get(tfl, remote_path)
        tfl.seek(0)
        return tfl.read()


    def put(self, local_file, remote_path, remote_mode=0644):
        """ Send a file to the remote system.

            :param local_file:  file-like object to be sent
            :param remote_path: path to the file (including file name) 
                                on the remote system
            :return:            None 
        """
        local_file.seek(0, os.SEEK_END)
        size = local_file.tell()
        local_file.seek(0)

        self._logger.info("Sending file [%s], mode %s, %d bytes, to device" 
                            % (remote_path, remote_mode, size))
        chan = self._session().scp_send(remote_path, remote_mode, size)
        chan.write(local_file.read())
        chan.close()
        self._logger.info("File [%s] transmission successful" % remote_path)  


def main():
    """ Standalone function; only defined if the class is run by itself. 
        This function uses some basic capabilities of the class. It is
        intended to be used for interactive testing during development,
        and as a showcase on how to use the class. 
    """
    if len(sys.argv) != 4:
        print "Usage: %s <host> <username> <password>" % sys.argv[0]
        sys.exit()
    ssh = SshConn(logger.init(), sys.argv[1], (sys.argv[2], sys.argv[3]))
    print ssh.cmd("echo $PATH")[1]
    print ssh.cmd("ls /")[1]
    print ssh.cmd("uptime")[1]
#    print ssh.cat("/etc/resolv.conf")
#    print ssh.cmd("dd if=/dev/zero of=/tmp/z bs=1024 count=1")[1]
#    print ssh.cmd("depmod")[1]
#    print ssh.cmd("nandwrite")[1]
#    print ssh.cmd("flash_erase")[1]
#    tfl = tempfile.TemporaryFile()
#    tfl.write("This is an example file transmission.")
#    ssh.put(tfl, "/home/root/test")
#    print ssh.cat("/home/root/test")


if __name__ == '__main__':
    main()



#!/usr/bin/python -tt
# -*- coding: utf-8 -*-
#
# MONK automated test framework module API
#
# Copyright (C) 2012 DResearch Fahrzeugelektronik GmbH
# Written and maintained by Thilo Fromm <fromm@dresearch-fe.de>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version
# 2 of the License, or (at your option) any later version.
#



# MONK automated integration test framework public API

from devicetestcase import DeviceTestCase
from device         import Device
from connection     import Connection
from ssh_conn       import SshConn
from serial_conn    import SerialConn
from bcc            import Bcc



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Package for MONK integration test case generic test classes
"""

import unittest
import datetime
import device
import threading
import logging

def config_logging():
    """ Configure the root logger """
    logger = logging.getLogger()
    logger.setLevel(logging.DEBUG)

    filename = "run-%s.log" % datetime.datetime.now().strftime("%Y%m%d_%H%M%S")

    handler = logging.FileHandler(filename, mode='w+')
    handler.setFormatter(logging.Formatter(
        "%(asctime)s %(levelname)s %(filename)s::%(funcName)s(): "
        + "%(message)s"))
        
    logger.addHandler(handler)


class DeviceTestCase(unittest.TestCase):
    """ This class is the base class for all MONK tests. It takes
        care of basic device initialization and guarantees that there's only
        one instance of :py:class:`Device` active at any time. 
        
        Classes inheriting from :py:class:`DeviceTestCase` will automatically
        win a logger object and a device object upon initialization. These two
        instance objects are provided by :py:class:`DeviceTestCase` :

        .. code-block:: python
                
                import devicetestcase

                class MyTest(devicetestcase.DeviceTestCase):

                      ....

                    def setUp(self):
                        self.dev.wait_for_network()
                        self.logger.debug("Device IP is: %s." % self.dev.host)

                      ....
    """

    __dev = None
    __devsem = threading.Lock()
    
    
    @classmethod
    def get_device(cls, devicetype, nand_boot=True, init_logging=True):
        """ Get the :py:class:`Device` singleton instance. A new instance will
            be created if none is available yet. 
        """
        if not cls.__dev:
            cls.__devsem.acquire()
            if not cls.__dev:
                cls.__create_device(devicetype, nand_boot, init_logging=init_logging)
            cls.__devsem.release()
        return cls.__dev


    @classmethod        
    def __create_device(cls, devicetype, nand_boot=True, init_logging=True):
        """ boot HidaV-device to NAND """
        if init_logging:
            config_logging()
        cls.__dev = device.Device( devtype = devicetype )
        if nand_boot:
            logging.getLogger(__name__).debug("Boot to NAND ...")
            cls.__dev.reboot(to_nand=True)


    def __init__(self, devicetype, *args, **kwargs): # pragma: no cover
        """ The class will create and add to self logger and dev objects upon
            instantiation.
        """
        init_logging = True
        if 'init_logging' in kwargs:
            init_logging = kwargs["init_logging"]
            del kwargs["init_logging"]
        super(DeviceTestCase, self).__init__(*args, **kwargs)
        self.dev = DeviceTestCase.get_device(devicetype, init_logging=init_logging)
        self.logger = logging.getLogger(__name__)
        



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# MONK automated test framework abstract connection 
#
# Copyright (C) 2012 DResearch Fahrzeugelektronik GmbH
# Written and maintained by Thilo Fromm <fromm@dresearch-fe.de>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version
# 2 of the License, or (at your option) any later version.
#

""" Package for the connection class """

import os
import logging
import serial_conn, ssh_conn, re

class Connection(object):
    """ Connection to a device.

        The connection class abstracts a device connection via serial and/or IP.
        It implements device command processing in a request/response manner.
    """

    def __init__(self, serial_setup = (os.getenv("MONK_CONSOLE_PORT", "/dev/ttyUSB1"), 115200, 8, 'N', 1, 1), 
                 network_setup = ( None, "eth0" ), login = ( "root", "" ),
                 boot_prompt = "HidaV boot on", serial_skip_pw = True,
                 reset_cb = None):
        """ Initialize a new connection instance.

            :param serial_setup: set of init options for the serial backend:
                ("port", baudrate, data bits, parity, stop bits, timeout_sec)
            :param network_setup: set of init options for network connections:
                ("ip or hostname", "network dev ON THE DEVICE") 
            :param login: tuple of (username, pass) for the device
            :param boot_prompt: string for identifying the boot loader prompt
            :param serial_skip_pw: True if the serial login does not prompt
                for a password
            :param reset_cb: callback to run when resetting the device
        """
        self._logger = logging.getLogger(__name__)
        self._login = login
        self._target_if = network_setup[1]
        self._serial = self._serial_setup(*serial_setup, 
                            skip_pass = serial_skip_pw, 
                            boot_prompt = boot_prompt,
                            reset_cb = reset_cb)
        if network_setup[0]:
            self.host = network_setup[0]


    @property
    def _ssh(self):
        """ SSH connection property. Will be created on the fly as required."""
        try:    
            return self.__ssh
        except AttributeError:
            self.__ssh = ssh_conn.SshConn( # pylint: disable-msg=W0201
                    self.host, self._login ) 
        return self.__ssh
    
    
    @_ssh.deleter # new-style properties unknown to pylint: disable-msg=E1101
    def _ssh(self): # pylint: disable-msg=E0102
        """ SSH connection property deleter. Deletes the property. """
        del self.__ssh

    
    def _serial_setup(self, port, baud, byte, parity, stop, timeout_sec, 
                      skip_pass, boot_prompt, reset_cb):
        """ Set up the serial communication back-end. """
        self._logger.debug("(re)opening serial port %s" % port)
        ser = serial_conn.SerialConn(
                        login = self._login, skip_pass = skip_pass, 
                        boot_prompt = boot_prompt,
                        reset_cb = reset_cb)
        ser.port     = port
        ser.baudrate = baud
        ser.bytesize = byte
        ser.parity   = parity
        ser.stopbits = stop
        ser.timeout  = timeout_sec
        ser.open()
        self._logger.debug("%s is now open." % port)
        return ser
 

    @property
    def host(self): # pylint: disable-msg=E0202
        """ The Host property. This can be set via init using network_setup,
            or it might be determined automatically at run-time by reading the
            IP address from the device using the serial connection.
        """
        try: 
            return self._host
        except AttributeError:
            ret = self._serial.cmd( 
                    r"ifconfig | grep -A 1 '" + self._target_if + r"'"
                  + r"| grep 'inet addr:' "
                  + r"| sed -e 's/.*inet\ addr:\([0-9.]\+\)\ .*/\1/'")[1]
            try:
                self._host = re.search( 
                    r'(?P<ip>[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3})'
                        , ret).group("ip")
            except AttributeError:
                raise Exception(
                    "Unable to collect device's IP address via serial port %s (got [%s])"
                    % (self._serial.port, ret))
        return self._host


    @host.setter # new-style properties unknown to pylint: disable-msg=E1101
    def host(self, value): # pylint: disable-msg=E0102,E0202
        """ Set the Host property. """
        self._host = value # pylint: disable-msg=W0201


    @host.deleter # new-style properties unknown to pylint: disable-msg=E1101
    def host(self): # pylint: disable-msg=E0102,E0202
        """ Delete the Host property. """
        try:
            del self._host
        except AttributeError:
            pass


    def cmd(self, cmd):
        """ Run a remote command on the device. 

            The command will be issued via SSH if the host name or IP address is 
            known to the connection (i.e. C{has_networking()} returns True),
            and via serial line otherwise.

            :param cmd:  command to be issued
            :return:     tuple of (retcode, command_output) 
        """
        self._logger.info("Ecexuting remote command [%s]" % cmd)
        try: 
            return self._ssh.cmd(cmd)
        except:
            self._logger.exception("Command execution of [%s] via SSH failed"
                    % cmd)
            self._logger.info("Falling back to serial for cmd [%s]" % cmd)
            return self._serial.cmd(cmd)


    def has_networking(self):
        """ (Actively) Check whether we currently have networking, i.e.
            whether the address of the remote device is known.
            This will trigger extraction of the IP address via serial
            if the address is currently unknown.
        """
        del self.host
        try:    
            self.host = self.host
        except: 
            return False
        return True
    
    
def main():   #pragma: no cover
    """ Standalone function; only defined if the class is run by itself. 
        This function uses some basic capabilities of the class. It is
        intended to be used for interactive testing during development,
        and as a showcase on how to use the class. 
    """
    import sys, time
    if len(sys.argv) < 3:
        print "Usage: %s <username> <password> [<ip address>]" % sys.argv[0]
        sys.exit()

    import bcc
    b = bcc.Bcc()

    conn = Connection( network_setup = (None, "eth1"), 
                        login = (sys.argv[1], sys.argv[2]) )

    print "Waiting for Networking to come up..."
    while not conn.has_networking():
        time.sleep(1)
        sys.stdout.write(".")
    print "\nWe now have networking."

    print "Executing ls /"
    print conn.cmd("ls /")[1]

    print "Now shutting down the device."
    b.heartbeat = 30
    conn._serial._reset_cb = None
    conn._serial.reboot()


if __name__ == '__main__':  #pragma: no cover
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# MONK automated test framework board controller handling
#
# Copyright (C) 2012 DResearch Fahrzeugelektronik GmbH
# Written and maintained by Thilo Fromm <fromm@dresearch-fe.de>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version
# 2 of the License, or (at your option) any later version.
#

""" Package for the board controller class """


import sys
import subprocess
import re
import threading
import time
import atexit
import logging
import os

class Bcc(object):
    """ The Board Controller class.

        This class represents the board controller of a connected device.
        The BCC is supposed to be connected to a host PC via its external serial 
        port / FTDI cable. 
    """
    
    def __init__(self, port = os.getenv("MONK_BCTRL_PORT", "/dev/ttyUSB0"), speed = 57600, drbcc = "drbcc"):
        """ Create a new BCC instance.

            :param port:  serial port to use for communication with the board
                          controller, defaults to content of environment
                          variable MONK_BCTRL_PORT or "/dev/ttyUSB0" if the
                          variable is not set
            :param speed: serial port speed
            :param drbcc: path to drbcc binary

            :raise: :py:exc:`OSError` if the drbcc binary was not found.
        """
        self.logger = logging.getLogger(__name__)
        self.__drbcc   = drbcc
        self.__port    = port
        self.__port_br = speed

        atexit.register(self.__cleanup)
        self.poweron()


    def __cleanup(self):
        """ Restore function to disable debug mode set in __init__. """
        try:
            # stop faking ignition
            self.cmd("debugset 16,00")
        except:
            pass


    def cmd(self, cmd="gets"):
        """ Execute a BCC command.

            :param cmd: command to be executed
            :return:    tuple of (exit_code, text_result): exit code and textual 
                        result of command execution as returned by drbcc
            :raise:     :py:exc:`OSError` if the drbcc binary was not found
        """
        text=""; rc = 0
        try:
            text = subprocess.check_output( 
                [self.__drbcc, 
                 "--dev=%s,%d" % (self.__port, self.__port_br),
                 "--cmd=%s" % (cmd, )],
                stderr = subprocess.STDOUT)
        except subprocess.CalledProcessError, e:
            rc = e.returncode
            text = e.output
        return rc, text


    def reset(self):
        """ Reset the system.
            
            This method will reset the system by calling :py:meth:`poweroff`, 
            sleeping for a second, then calling :py:meth:`poweron`.
        """
        self.poweroff()
        time.sleep(1)
        self.poweron()


    def poweron(self):
        """ Power ON the system.

            This method will fake the ignition pin to 'ON', then set
            watchdog :py:meth:`heartbeat` to the maximum value. 
        """
        self.logger.debug("poweron")
        self.cmd("debugset 16,0100010001")
        self.heartbeat = 65535
        self.hddpower = True


    def poweroff(self):
        """ Power ON the system.

            This method will fake the ignition pin to 'OFF', then set 
            watchdog :py:meth:`heartbeat` to zero. 
        """
        self.logger.debug("poweroff")
        self.cmd("debugset 16,0100000001")
        self.heartbeat = 0
        self.hddpower = False


    @property
    def status(self):
        """ BCC status information property.

            :return: 4 lines of text containing detailed bcc status, or the
                     empty string if the request failed.
        """
        return self.cmd("gets")[1]


    def __status(self, what):
        """ Helper method to extract several status values

            :param what: status property, e.g. "HDD-Power" or "Ignition"
            :return:     string value representing the field's status
        """
        s = re.search(what + r': (?P<stat>\w+)',
                       self.status,
                    ).group('stat')
        return s


    @property
    def ignition(self):
        """ BCC ignition state property.

            :return: True or False
        """
        i  = self.__status("Ignition")
        return (i == "on")


    @property
    def hddpower(self):
        """ BCC harddisk power state property.

            :param hddpower: can be set to True or False, i.e. whether 
                             to power the HDD or not.
            :return:         True or False
        """
        h  = self.__status("HDD-Power")
        return (h == "on")


    @hddpower.setter
    def hddpower(self, power):
        """" Setter for the hddpower property. """
        ps = "1" if power else "0"
        self.cmd("hdpower " + ps)


    @property
    def heartbeat(self):
        """ BCC heartbeat state property.

            :param heartbeat: number of seconds before RESET
            :return:          tuple of (curr, max) representing the maximum count
                              upon which a RESET will be triggered as well as the 
                              current count (in seconds).
        """
        text = self.cmd("debugget 1")[1]
        values = re.search(r'data: (?P<v1>[0-9A-F]+) (?P<v2>[0-9A-F]+) '
                                + r'(?P<m1>[0-9A-F]+) (?P<m2>[0-9A-F]+)',
                            text)

        current = int( values.group("v1") + values.group("v2"), 16)
        maximum = int( values.group("m1") + values.group("m2"), 16)

        return current, maximum


    @heartbeat.setter
    def heartbeat(self, seconds):
        """" Setter for the heartbeat property. """
        self.cmd("heartbeat %s" % seconds)


def main(): #pragma: no cover
    """ Standalone function; only defined if the class is run by itself. 
        This function uses some basic capabilities of the class. It is
        intended to be used for interactive testing during development,
        and as a showcase on how to use the class. 
    """
    b = Bcc()

    if len(sys.argv) > 1:
        for cmd in sys.argv[1:]:
            if cmd == "reset":
                b.reset()
                continue
            if cmd == "off":
                b.poweroff()
                continue
            if cmd == "on":
                b.poweron()
                continue
            ret, txt = b.cmd(cmd)
            print "RETURN VALUE %s" %ret
            print txt
        sys.exit()

    while True:
        banner = "~*~AN~*~" if b.ignition   \
             else "___AUS___"

        subprocess.call("clear")
        subprocess.call(["banner", banner])

        print b.status
        c, m = b.heartbeat
        print "Heartbeat timeout: %s seconds of %s total" % (c, m)
        time.sleep(1)


if __name__ == '__main__': #pragma: no cover
    main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# MONK automated test framework serial connection handling
#
# Copyright (C) 2012 DResearch Fahrzeugelektronik GmbH
# Written and maintained by Thilo Fromm <fromm@dresearch-fe.de>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version
# 2 of the License, or (at your option) any later version.
#

""" Package for the Serial Connection class """

import serial
import re
import urllib
import time
import sys
import logging

import bcc

LAST_LINE_UPON_SHUTDOWN = "Detaching DM devices" 

class SerialConn(serial.Serial):
    """Serial connection to a device.
       The serial class implements device access via the serial port.

       FIXME: *args nach benannten Parametern funktioniert nicht. Lösung bereits unsauber implementiert.
       FIXME: bei Problemen im Login kann es sein, dass die while Loop in __wait_for_known_boot_state() nicht aufhört!
    """
    
    def __init__(self, login = ("root", ""), 
                 skip_pass = True, boot_prompt="HidaV boot on", 
                 reset_cb = None, args=None, kwargs=None):
        """Initialize a new instance of the serial communication class.

           :param login:       tuple of (username, pass)
           :type login:        set of two members
           :param skip_pass:   True if the login does not prompt for a password
           :param boot_prompt: the prompt to identify the boot loader with
           :param reset_cb:    custom RESET callback to run when the device
                               is rebooted
        """
        self._logger = logging.getLogger(__name__)
        self.__args = args if args is not None else []
        self.__kwargs = kwargs if kwargs is not None else {}
        try:    
            super(SerialConn, self).__init__(*self.__args, **self.__kwargs)
        except AttributeError: 
            pass
        self._login = login
        self._skip_pass = skip_pass
        self._boot_prompt = boot_prompt
        self._reset_cb = reset_cb


    def read_until (self, target, trigger_write="\n", timeout=None):
        """ Read up to a trigger text, then stop.

            :param target: target string to match
            :param trigger_write: 
                string to be sent via serial if a read timeout occurs
            :param timeout: custom timeout for :py:obj:`trigger_write`
            :return: all text read up to the point where :py:obj:`target` 
                appeared, including :py:obj:`target`
        """
        self._logger.debug("reading 'til [%s], triggering output with [%s]" 
                % (urllib.quote(target), urllib.quote(trigger_write)))
        buf      = ""
        log_line = ""
        if timeout:
            old_to = self._timeout
            self._timeout = timeout
        while not target in buf:
            ret = self.read()
            if ret == "":
                self._logger.debug("Triggering with [%s] target [%s]" 
                                        % (urllib.quote(trigger_write), urllib.quote(target)))
                time.sleep(0.25)
                self.write( trigger_write )
            else:
                buf += ret
                if ret == '\n':
                    self._logger.debug("[%s]"  % log_line.strip())
                    log_line = ""
                else:
                    log_line += ret
        self._logger.debug("Got it: [%s]" % urllib.quote(target))
        if timeout:
            self._timeout = old_to
        return buf


    def dump_receive_buffer(self):
        """ Return the contents of the serial input buffer """
        return self.read(self.inWaiting())


    def readline_until(self, target, trigger_write="\n"):
        """ Read up to a trigger text, read the whole line, then stop.

            This method works just like :py:meth:`read_until` with the exception 
            that it will read the whole last line containing the target string.
            
            :param target:        target string to match
            :param trigger_write: string to be sent via serial if a read timeout 
                occurs
            :return:              all text read up to the point where 
                :py:obj:`target` appeared, including :py:obj:`target`
        """
        buf = self.read_until( target, trigger_write )
        buf += self.readline()
        return buf


    @property
    def __boot_state(self):
        """ The current system state.

            :return: string "bootloader", "login", "shell", or "UNKNOWN"
        """
        buf = self.read_until("\n", timeout=3)

        ret = "shell"  if self._login[0] + "@" in buf \
                        else "login" if "login:" in buf \
                        else "bootloader" if self._boot_prompt in buf \
                        else "UNKNOWN"
        self._logger.debug("Current system state is %s" % ret)

        return ret


    def __wait_for_known_boot_state(self):
        """ Wait for a KNOWN boot state. """
        self._logger.debug("Waiting for a known system state...")

        state = "UNKNOWN"
        while state == "UNKNOWN":
            state = self.__boot_state

        self._logger.debug("Got system state %s" % state)
        return state


    def login(self):
        """ Log in to a device.

            This method will log in to a device. It will check wheter we are 
            already logged in (in which case the method succeeds early), and 
            whether we're currently in the boot loader (in which case the 
            method will boot the device, then log in).
        """
        state = self.__wait_for_known_boot_state();

        if state == "shell":
            self._logger.debug("Already logged in.")
            return

        if state == "bootloader":
            self._logger.debug("System has stopped at"
                                + " bootloader; booting...")
            # the bootloader loses bytes on the serial line, so repeat this 
            self.write("\nboot\n")
            self.flush()
            self.write("\nboot\n")
            self.flush()
            self.write("\nboot\n")
            self.flush()

        self._logger.info("Login for user %s." % self._login[0])
        self.flushInput()
        self.flushOutput()

        self.read_until("login:", "\n", timeout=20)
        self.write(self._login[0] + "\n")

        if not self._skip_pass:
            self._logger.debug("Sending password for user %s." 
                                    % self._login[0])
            self.read_until("Password:")
            self.write(self._login[1] + "\n")

        self.read_until(self._login[0] + "@")
        # make sure no kernel messages go to the serial console while
        # we are remote-controlling the device
        self.write('echo "0 0 0 0" > /proc/sys/kernel/printk\n')


    def __run_cmd(self, cmd):
        """ Generic command executor for both boot loader and linux shell cmds.

            This method will run a command, wrapping the command's output into
            boundaries for later extraction.

            :param cmd: command to execute
            :return:    string buffer containing command output
        """
        self.flushInput()
        self.flushOutput()

        self.read_until("\n", timeout=30)
        
        self.write(   r'remote_trigger="OUTPUT";'
                    + r'echo "===${remote_trigger} STARTS===";' 
                    + cmd 
                    + r';echo "===${remote_trigger} ENDS==="' + "\n")

        buf = self.readline_until("===OUTPUT ENDS===")

        start = buf.index("===OUTPUT STARTS===")
        start = buf.index("\n", start)

        end = buf.index("===OUTPUT ENDS===")
        end = buf.rindex("\n", 0, end)

        return buf[start:end].strip()


    def cmd(self, cmd):
        """ Execute a linux user space command on the remote system.
            This method will execute a command on the remote system,
            returning the retcode of that command and all the output
            which appeared on the serial console while the command was
            executed.

            .. warning:: 
                The command to be run MUST be guaranteed to return; 
                so issuing "halt" or "reboot" is a bad idea. Also, Linux
                user space is required for execution (e.g. when calculating
                the return value of the command executed). :py:meth:`cmd` does 
                not currently work with the boot loader prompt.

            :param cmd: the command to run
            :return:    tuple of (retcode, command output) 
        """
        cmd = cmd.strip() + r'; echo -e "\nRETCODE=$?"'
        self._logger.info("Executing command [%s]" % cmd)

        self.login()
        buf = self.__run_cmd(cmd)

        rcd = int(re.search(r'RETCODE=(?P<retcode>[0-9]+)', 
                        buf).group("retcode"))

        self._logger.info("Command [%s] ret: #%d" % (cmd, rcd))
        return rcd, buf


    def logout(self):
        """ Log out of the system. """
        if self.__boot_state == "shell":
            self._logger.debug("Logging out.")
            self.write("\nexit\n")
            self.read_until("login:")
    
    
    def reboot(self, reboot_cmd="halt", sync=False, stop_at_bootloader=False):
        """ Reboot the system.
        
            This method will reboot the system, kicking it with a HW reset if the 
            serial class was asked to do so at instantiation time.
            
            By default the method will issue a reboot command and then return 
            immediately.

            :param reboot_cmd: actual command to trigger the reboot; 
                Defaults to "halt" since most of our systems are self-resetting :)

            :param sync: Wait for the reboot to happen; return only after the 
                device rebooted and a login is available. Defaults to False.

            :param stop_at_bootloader: Reboot, then wait for boot loader 
                messages on the serial line, then interrupt the boot loader. 
                This function returns after we got a boot loader prompt.

            :return: All the console messages from when the reboot has been 
                issued up to the point the method stopped reading. May be the 
                empty string if an asynchronous reboot has been requested.
        """
        self._logger.info("Rebooting %s" % ("and stopping at bootloader" if
                stop_at_bootloader else "synchronously" if sync else "."))

        state = self.__wait_for_known_boot_state()

        buf = ""
        if state in ("shell", "login"):
            self.login()
            self.write("\n" + reboot_cmd + "\n")
            if self._reset_cb:
                buf += self.readline_until(LAST_LINE_UPON_SHUTDOWN)
                self._reset_cb()

        if stop_at_bootloader:
            buf += self.read_until(self._boot_prompt, ".\n", 0.001)
        elif sync:
            buf += self.readline_until("login:")
        self._logger.info("OK")
        return buf


    def boot_to_nand(self, kernel_partition = None, rootfs_partition = None,
                            sync = False):
        """ Reboot the device to NAND.

            :param kernel_partition: 
                kernel partition number to boot into.
            :param rootfs_partition: 
                root fs parition number to boot into.
            :param sync:  
                Wait for the reboot to happen; return only after the device 
                rebooted and a login is available. Defaults to False.
            :return:            
                All the console messages from when the reboot has been issued up 
                to the point the method stopped reading.
        """
        buf = ""
        if kernel_partition != None:
            kernel_offset = "0x200000" if kernel_partition == 2 else "0xC00000"
            self._logger.info("Rebooting to NAND" 
                               + " (kernel: mtd%s, rootfs: mtd%s)."
                               % (kernel_partition, rootfs_partition))

        buf += self.reboot(sync=True, stop_at_bootloader=True)

        self.flushInput()
        self.flushOutput()
        self.flush()
        time.sleep(1)

        if kernel_partition != None:
            self._logger.debug("Setting kernel offset to %s." 
                               % kernel_offset)
            self.write("\nsetenv kernel_offset %s\n" % kernel_offset)
            self.flush()
            time.sleep(1)

        if rootfs_partition != None:
            self._logger.debug("Setting kernel root to /dev/blockdev%s." 
                               % rootfs_partition)
            self.write("\nsetenv rootfs_device /dev/blockdev%s\n" 
                        % rootfs_partition)
            self.flush()
            time.sleep(1)

        self._logger.debug("Issuing 'run bootnand'.")
        self.write("\nrun bootnand\n")
        self.flush()
        time.sleep(1)
        if sync:
            buf += self.readline_until("login:")
        self._logger.debug("Now rebooted to NAND.")

        return buf


def main():
    """ Standalone function to run the class all by itself.

        This function uses some basic capabilities of the class. It is
        intended to be used for interactive testing during development,
        and as a showcase on how to use the class. 
    """
    b = bcc.Bcc()
    if len(sys.argv) < 3:
        print "Usage: %s <username> <password> [<command>]" % sys.argv[0]
        sys.exit()
    scn = SerialConn(logger.init(), (sys.argv[1], sys.argv[2]),
            reset_cb=b.reset)
    scn.port     = "/dev/ttyUSB1"
    scn.baudrate = 115200
    scn.bytesize = 8
    scn.parity   = 'N'
    scn.stopbits = 1
    scn.timeout  = 1
    scn.open()

    if len(sys.argv) > 3:
        for cmd in sys.argv[3:]:
            print "\n ######## "
            print " ######## Running custom command >>>> %s <<<<" % cmd
            print " ######## \n"
            retcode, bla = scn.cmd(cmd)
            print "\n\n#######\nRETCODE: %s, EXEC MSGS:\n%s" % (retcode, bla)
        sys.exit()

    print " ######## "
    print " ######## ls /"
    print " ######## "
    print scn.cmd("ls /")[1]
    print " ######## "
    print " ######## Rebooting..."
    print " ######## "
    bootlog = scn.reboot(sync = True)
    print " ######## "
    print " ######## Reboot OK"
    print " ######## \n"
    print "   --- reboot messages ---  \n"
    print bootlog
    print "   --- ------ -------- ---  \n"
    print " ######## "
    print " ######## Rebooting into bootloader..."
    print " ######## "
    bootlog = scn.reboot(stop_at_bootloader = True)
    print " ######## Device stopped in boot loader"
    print "   --- reboot messages ---  \n"
    print bootlog
    print "   --- ------ -------- ---  \n"
    print scn.dump_receive_buffer()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# MONK automated test framework high-level device functions
#
# Copyright (C) 2012 DResearch Fahrzeugelektronik GmbH
# Written and maintained by Thilo Fromm <fromm@dresearch-fe.de>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version
# 2 of the License, or (at your option) any later version.
#

""" Package for the device class """

import time
import atexit
import sys
import logging

import connection
import bcc


class Device(object):
    """ This class abstracts access to a device, i.e. one single
        physical system.

        Bsp: 
            >>> import device
            >>> d = device.Device()
            >>> d.wait_for_network()
            login
            True
            >>> d.conn.host
            '172.29.21.174'

        FIXME bcc integrieren
    """

    DEVICE_TYPES = {
        "hidav": {
            "name"          : "HidaV",
            "network_setup" : (None, "eth1"),
            "login"         : ("root", ""),
            "boot_prompt"   : "HidaV boot on",
            "reset_cb"      : True,
            "serial_skip_pw": True },
        "hipox": {
            "name"          : "Hipox",
            "network_setup" : (None, "eth1"),
            "login"         : ("root", "hydra01"),
            "boot_prompt"   : "$ ",
            "reset_cb"      : False,
            "serial_skip_pw": False },
    }


    def __init__(self, devtype):
        """ Initialize a device instance.

            :param devtype: Device type, either "hidav" or "hipox"
        """
        try:
            self._setup = Device.DEVICE_TYPES[devtype.lower()]
        except KeyError:
            raise Exception("Unknown device type %s." % devtype)

        self.bcc = bcc.Bcc()
        rst = self.bcc.reset if self._setup["reset_cb"] == True else None
        atexit.register(self.__shutdown)

        self.conn = connection.Connection(
            network_setup = self._setup["network_setup"],
            login = self._setup["login"],
            serial_skip_pw = self._setup["serial_skip_pw"],
            reset_cb = rst)
        self._logger = logging.getLogger(__name__)


    def __shutdown(self):
        """ Shut down the device.

            This function is atexit() registered. It shuts down the device by
            setting a low WD timeout, then issuing reboot() via serial.
        """
        try:
            self._logger.debug( "Shutting down the device." )
            self.conn._serial._reset_cb = None
            self.conn._serial.reboot()
            self.bcc.heartbeat = 10
            del self.bcc
        except:
            # Interpreter Shutdown Time Pokemon Exception Handling (ISTPEH):
            # gotta catch them all!
            pass


    def reboot(self, to_nand = False):
        """ Reboot the device. Return after reboot was successful.

            Optionally reboot to NAND flash into the currently active
            kernel / root fs combination (see bootconfig).

            :param to_nand: reboot into currently active NAND partitions
            :return:        buffer containing all the messages from the reboot
        """
        if self._setup["name"] == "HidaV" and to_nand:
            self._logger.debug("Benutze Hidav und nand true...")
            return self.conn._serial.boot_to_nand(sync=True )

        return self.conn._serial.reboot(sync=True)


    def wait_for_network(self, max_wait=120):
        """Wait for networking to come up.

            :param max_wait: max time in seconds to wait
            :return:         True if networking is available,
                False if timeout occurred.
        """
        self._logger.debug("Waiting for Networking to come up...")
        while not self.conn.has_networking():
            time.sleep(1)
            self._logger.debug("wait {0}s".format(max_wait))
            max_wait -= 1
            if max_wait == 0:
                break
        return (max_wait > 0)


    @property
    def firmware_version(self):
        """ Firmware version property. This is the device's lsb_version
            revision string.
        """
        try:
            return self._fw_version
        except AttributeError:
            self._fw_version = self.conn.cmd(  # pylint: disable-msg=W0201
                    "lsb_release -r | awk '{ print $2 }'" )[1].strip()
            return self._fw_version


    @firmware_version.deleter     # pylint: disable-msg=E1101
    def firmware_version(self): # pylint: disable-msg=E0102
        """ Firmware version property deleter. """
        del self._fw_version


    # FIXME bootconfig property is HidaV specific and should be refactored into a devtype specific class
    @property
    def bootconfig(self):
        """ Bootconfig property. This is a dictionary representing the device's
            boot partition configuration.
            E.g.

                {"epoch": 42, "kernel": 2, "rootfs": 5}

            would indicate the current kernel is on mtd2, the current rootfs on
            mtd5, and the entry is the 42nd ever written.
        """
        try:
            return self._bootconfig
        except AttributeError:
            ret = self.conn.cmd(  # pylint: disable-msg=W0201
                    """bootconfig | """
                +   """awk '/kernel/{ k=$2 } /rootfs/{ r=$2 } """
                +       """ /epoch/{ e=$2 } END{ print k " " r " " e }'"""
                        )[1].split(" ")
            try:
                self._bootconfig = { "kernel": int(ret[0]),
                                     "rootfs": int(ret[1]),
                                     "epoch" : int(ret[2])  }
            except:
                self._logger.warn("Unable to determine boot configuration.")
                self._bootconfig = { "kernel": 2, "rootfs": 4, "epoch": 0 }
            return self._bootconfig


    @bootconfig.setter              # pylint: disable-msg=E1101
    def bootconfig(self, value):  # pylint: disable-msg=E0102,E0202
        """ Set the bootconfig property (kernel and rootfs mtd; not the epoch).

            :param value: dictionary containing kernel and rootfs partition
                number to be set, e.g.  {"kernel": 2, "rootfs": 5}
        """
        del self.bootconfig
        if value["kernel"] != self.bootconfig["kernel"]:
            self._logger.info("Setting kernel partition to mtd%s"
                                % value["kernel"])
            self.conn.cmd("bootconfig set-kernel mtd%d" % value["kernel"])

        if value["rootfs"] != self.bootconfig["rootfs"]:
            self._logger.info("Setting rootfs partition to mtd%s"
                                % value["rootfs"])
            self.conn.cmd("bootconfig set-rootfs mtd%d" % value["rootfs"])
        del self.bootconfig


    @bootconfig.deleter     # pylint: disable-msg=E1101
    def bootconfig(self): # pylint: disable-msg=E0102
        """ Bootconfig property deleter. """
        try:
            del self._bootconfig
        except AttributeError:
            pass


    def update_package_index(self):
        """ Update the device's package index remotely.

            :raise: Exception if the update failed.
        """
        self._logger.debug("Updating the package index...")
        ret, msgs = self.conn.cmd("opkg update")
        if ret != 0:
            raise Exception("Updating the package index failed with #%s:\n%s"
                    % (ret, msgs))


    def install_package(self, package_name, force = False):
        """ Install a package on the device. The package index will be updated
            prior to the installation.

            :param package_name: name of the package to be installed
            :param force:        force re-installation of already installed 
                                 package
            :raise:              exception if the install failed.
        """
        f = "--force-reinstall" if force else ""
        self._logger.info("Installing package %s %s."
                            % (package_name, f) )
        self.update_package_index()
        ret, msgs = self.conn.cmd("opkg install %s %s" % (f, package_name))
        if ret != 0:
            raise Exception("Installing package %s failed with #%s:\n%s"
                    % (package_name, ret, msgs))


    def remove_package(self, package_name):
        """ Remove a package from the device.

            :param package_name: name of the package to be removed.
            :raise:              exception if the install failed.
        """
        self._logger.info("Removing package %s." % package_name)
        ret, msgs = self.conn.cmd(
                "opkg remove --force-removal-of-dependent-packages %s"
                % package_name)
        if ret != 0:
            raise Exception("Installing package %s failed with #%s:\n%s"
                    % (package_name, ret, msgs))


def main():
    """ Standalone function; only defined if the class is run by itself.
        This function uses some basic capabilities of the class. It is
        intended to be used for interactive testing during development,
        and as a showcase on how to use the class.
    """
    dev = Device(devtype = "hipox")

    while not dev.bcc.ignition:
        print "Please switch on the device."
        time.sleep(1)

    print "Connecting to device..."
    print "Firmware version: %s" % dev.firmware_version

    print "Boot config:"
    print "  Kernel:      /dev/mtd%s" % dev.bootconfig["kernel"]
    print "  rootfs: /dev/blockdev%s" % dev.bootconfig["rootfs"]
    print "  epoch :             #%s" % dev.bootconfig["epoch"]

    print "Waiting for Networking to come up..."
    while not dev.conn.has_networking():
        time.sleep(1)
        sys.stdout.write(".")
    print "\nWe now have networking."

    print " Checking package install functionality:"
    print " ----------------------"
    print " Package 'zip' is not installed:"
    retc, msg = dev.conn.cmd("zip --help")
    print "Retcode: %s" % retc
    print msg
    print "----------------------"

    print " Installing zip"
    dev.install_package("zip")
    print "Now zip is  there:"
    retc, msg = dev.conn.cmd("zip --help")
    print "Retcode: %s" % retc
    print msg
    print "----------------------"

    print " Removing zip"
    dev.remove_package ("zip")
    print "Now zip is gone:"
    retc, msg = dev.conn.cmd("zip --help")
    print "Retcode: %s" % retc
    print msg
    print "----------------------"

    print "Now shutting down the device."


if __name__ == '__main__':
    main()
    



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# MONK Automated Testing Framework
#
# Copyright (C) 2013 DResearch Fahrzeugelektronik GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version
# 2 of the License, or (at your option) any later version.
#

from distutils.core import setup

project = 'monk_tf'
src_path = 'src'

setup(
    name=project,
    version='v0.1',
    description = 'a test framework for embedded systems',
    author = 'DResearch Fahrzeugelektronik GmbH',
    author_email = 'project-monk@dresearch-fe.de',
    url='https://github.com/DFE/MONK',
    packages=[project],
    package_dir = { '' : src_path }
)
